@@ -32,7 +32,6 @@
  *
  * @author Adeel Asghar <adeel.asghar@liu.se>
  *
- * RCS: $Id$
  *
  */
 
@@ -126,18 +125,15 @@ int VariablesTreeItem::columnCount() const
 
 bool VariablesTreeItem::setData(int column, const QVariant &value, int role)
 {
-  if (column == 0 && role == Qt::CheckStateRole)
-  {
-    if (value.toInt() == Qt::Checked)
+  if (column == 0 && role == Qt::CheckStateRole) {
+    if (value.toInt() == Qt::Checked) {
       setChecked(true);
-    else if (value.toInt() == Qt::Unchecked)
+    } else if (value.toInt() == Qt::Unchecked) {
       setChecked(false);
+    }
     return true;
-  }
-  else if (column == 1 && role == Qt::EditRole)
-  {
-    if (mValue.compare(value.toString()) != 0)
-    {
+  } else if (column == 1 && role == Qt::EditRole) {
+    if (mValue.compare(value.toString()) != 0) {
       mValueChanged = true;
       mValue = value.toString();
     }
@@ -148,11 +144,9 @@ bool VariablesTreeItem::setData(int column, const QVariant &value, int role)
 
 QVariant VariablesTreeItem::data(int column, int role) const
 {
-  switch (column)
-  {
+  switch (column) {
     case 0:
-      switch (role)
-      {
+      switch (role) {
         case Qt::DisplayRole:
           return mDisplayVariableName;
         case Qt::DecorationRole:
@@ -168,27 +162,26 @@ QVariant VariablesTreeItem::data(int column, int role) const
           return QVariant();
       }
     case 1:
-      switch (role)
-      {
+      switch (role) {
         case Qt::DisplayRole:
-          return mValue;
+        case Qt::ToolTipRole:
         case Qt::EditRole:
           return mValue;
         default:
           return QVariant();
       }
     case 2:
-      switch (role)
-      {
+      switch (role) {
         case Qt::DisplayRole:
+        case Qt::ToolTipRole:
           return mUnit;
         default:
           return QVariant();
       }
     case 3:
-      switch (role)
-      {
+      switch (role) {
         case Qt::DisplayRole:
+        case Qt::ToolTipRole:
           return mDescription;
         default:
           return QVariant();
@@ -766,27 +759,13 @@ VariablesWidget::VariablesWidget(MainWindow *pMainWindow)
 {
   setMinimumWidth(175);
   mpMainWindow = pMainWindow;
-  // create the find text box
-  mpFindVariablesTextBox = new QLineEdit;
-  mpFindVariablesTextBox->setPlaceholderText(Helper::findVariables);
-  connect(mpFindVariablesTextBox, SIGNAL(returnPressed()), SLOT(findVariables()));
-  connect(mpFindVariablesTextBox, SIGNAL(textEdited(QString)), SLOT(findVariables()));
-  // create the case sensitivity checkbox
-  mpFindCaseSensitiveCheckBox = new QCheckBox(tr("Case Sensitive"));
-  connect(mpFindCaseSensitiveCheckBox, SIGNAL(toggled(bool)), SLOT(findVariables()));
-  // create the find syntax combobox
-  mpFindSyntaxComboBox = new QComboBox;
-  mpFindSyntaxComboBox->addItem(tr("Regular Expression"), QRegExp::RegExp);
-  mpFindSyntaxComboBox->setItemData(0, tr("A rich Perl-like pattern matching syntax."), Qt::ToolTipRole);
-  mpFindSyntaxComboBox->addItem(tr("Wildcard"), QRegExp::Wildcard);
-  mpFindSyntaxComboBox->setItemData(1, tr("A simple pattern matching syntax similar to that used by shells (command interpreters) for \"file globbing\"."), Qt::ToolTipRole);
-  mpFindSyntaxComboBox->addItem(tr("Fixed String"), QRegExp::FixedString);
-  mpFindSyntaxComboBox->setItemData(2, tr("Fixed string matching."), Qt::ToolTipRole);
-  connect(mpFindSyntaxComboBox, SIGNAL(currentIndexChanged(int)), SLOT(findVariables()));
-  // expand all button
-  mpExpandAllButton = new QPushButton(Helper::expandAll);
-  // collapse all button
-  mpCollapseAllButton = new QPushButton(Helper::collapseAll);
+  // tree search filters
+  mpTreeSearchFilters = new TreeSearchFilters(this);
+  mpTreeSearchFilters->getSearchTextBox()->setPlaceholderText(Helper::findVariables);
+  connect(mpTreeSearchFilters->getSearchTextBox(), SIGNAL(returnPressed()), SLOT(findVariables()));
+  connect(mpTreeSearchFilters->getSearchTextBox(), SIGNAL(textEdited(QString)), SLOT(findVariables()));
+  connect(mpTreeSearchFilters->getCaseSensitiveCheckBox(), SIGNAL(toggled(bool)), SLOT(findVariables()));
+  connect(mpTreeSearchFilters->getSyntaxComboBox(), SIGNAL(currentIndexChanged(int)), SLOT(findVariables()));
   // create variables tree widget
   mpVariablesTreeView = new VariablesTreeView(this);
   mpVariablesTreeModel = new VariablesTreeModel(mpVariablesTreeView);
@@ -800,15 +779,12 @@ VariablesWidget::VariablesWidget(MainWindow *pMainWindow)
   // create the layout
   QGridLayout *pMainLayout = new QGridLayout;
   pMainLayout->setContentsMargins(0, 0, 0, 0);
-  pMainLayout->addWidget(mpFindVariablesTextBox, 0, 0, 1, 2);
-  pMainLayout->addWidget(mpFindCaseSensitiveCheckBox, 1, 0);
-  pMainLayout->addWidget(mpFindSyntaxComboBox, 1, 1);
-  pMainLayout->addWidget(mpExpandAllButton, 2, 0);
-  pMainLayout->addWidget(mpCollapseAllButton, 2, 1);
-  pMainLayout->addWidget(mpVariablesTreeView, 3, 0, 1, 2);
+  pMainLayout->setAlignment(Qt::AlignTop | Qt::AlignLeft);
+  pMainLayout->addWidget(mpTreeSearchFilters, 0, 0);
+  pMainLayout->addWidget(mpVariablesTreeView, 1, 0);
   setLayout(pMainLayout);
-  connect(mpExpandAllButton, SIGNAL(clicked()), mpVariablesTreeView, SLOT(expandAll()));
-  connect(mpCollapseAllButton, SIGNAL(clicked()), mpVariablesTreeView, SLOT(collapseAll()));
+  connect(mpTreeSearchFilters->getExpandAllButton(), SIGNAL(clicked()), mpVariablesTreeView, SLOT(expandAll()));
+  connect(mpTreeSearchFilters->getCollapseAllButton(), SIGNAL(clicked()), mpVariablesTreeView, SLOT(collapseAll()));
   connect(mpVariablesTreeModel, SIGNAL(rowsInserted(QModelIndex,int,int)), mpVariableTreeProxyModel, SLOT(invalidate()));
   connect(mpVariablesTreeModel, SIGNAL(rowsRemoved(QModelIndex,int,int)), mpVariableTreeProxyModel, SLOT(invalidate()));
   connect(mpVariablesTreeModel, SIGNAL(itemChecked(QModelIndex,qreal,int)), SLOT(plotVariables(QModelIndex,qreal,int)));
@@ -1330,20 +1306,21 @@ void VariablesWidget::showContextMenu(QPoint point)
   }
 }
 
+/*!
+ * \brief VariablesWidget::findVariables
+ * Finds the variables in the Variables Browser.
+ */
 void VariablesWidget::findVariables()
 {
-  QString findText = mpFindVariablesTextBox->text();
-  if (mpFindVariablesTextBox->text().isEmpty())
-  {
-    findText = "";
-  }
-  QRegExp::PatternSyntax syntax = QRegExp::PatternSyntax(mpFindSyntaxComboBox->itemData(mpFindSyntaxComboBox->currentIndex()).toInt());
-  Qt::CaseSensitivity caseSensitivity = mpFindCaseSensitiveCheckBox->isChecked() ? Qt::CaseSensitive: Qt::CaseInsensitive;
+  QString findText = mpTreeSearchFilters->getSearchTextBox()->text();
+  QRegExp::PatternSyntax syntax = QRegExp::PatternSyntax(mpTreeSearchFilters->getSyntaxComboBox()->itemData(mpTreeSearchFilters->getSyntaxComboBox()->currentIndex()).toInt());
+  Qt::CaseSensitivity caseSensitivity = mpTreeSearchFilters->getCaseSensitiveCheckBox()->isChecked() ? Qt::CaseSensitive: Qt::CaseInsensitive;
   QRegExp regExp(findText, caseSensitivity, syntax);
   mpVariableTreeProxyModel->setFilterRegExp(regExp);
   /* expand all so that the filtered items can be seen. */
-  if (!findText.isEmpty())
+  if (!findText.isEmpty()) {
     mpVariablesTreeView->expandAll();
+  }
 }
 
 void VariablesWidget::directReSimulate()
